/*******************************************************************************
Filename:       hal_mcu.c

Autor:          Onur Kayikci
                Texas Instruments Freising, Germany
Datum:          19.11.2009

Description:    This file contains the MSP430 device specific configurations.

Processor:      MSP430F5529

*******************************************************************************/


/**************************************************************************************************
 *                                           INCLUDES
 **************************************************************************************************/
#include "hal_mcu.h"
#include "msp430x552x.h"
#include "hal_types.h"


/**************************************************************************************************
 *                                          CONSTANTS
 **************************************************************************************************/



/* ------------------------------------------------------------------------------------------------
 *                                          Clock Type
 * ------------------------------------------------------------------------------------------------
 */
#define HAL_CLOCK_TYPE_DCO         1
#define HAL_CLOCK_TYPE_CRYSTAL     0
#define SYSCLK                     SYSCLK_4MHZ




/**************************************************************************************************
 *                                       FUNCTIONS - API
 **************************************************************************************************/

void Mcu_Clock_Init(void);
static void halBoardGetSystemClockSettings(uint8  systemClockSpeed,
                                           uint8  *setDcoRange,
                                           uint8  *setVCore,
                                           uint16 *setMultiplier);
static void halBoardSetVCoreUp   (uint8 VCore);
static void halBoardSetVCoreDown (uint8 VCore);
static void halBoardSetVCore     (uint8 VCore);
void SPI_A0_Init(void);
void SPI_A0_Send_Data(uint8* DATA, uint8 i);

/**************************************************************************************************
 * @fn      mcu_clock_init
 *
 * @brief   Initilize Clock
 *
 * @param   None
 *
 * @return  None
 **************************************************************************************************/

void Mcu_Clock_Init(void)
{
  /* The DCO library does not handle the stack */    
  /* frame well. As a result, this macro must */     
  /* be the first thing that runs after main. */     
  /* Disable watchdog timer */                       
  WDTCTL = WDTPW | WDTHOLD;                          
                                                     
  if ( HAL_CLOCK_TYPE_CRYSTAL)                   
  {                                                  
    /* Configure external crystal TBD */             
  }                                                  
  if ( HAL_CLOCK_TYPE_DCO)                  
  {                                                  
    /* Note that when halBoardSetSystemClock() */    
    /* is changed, the HAL_CPU_CLOCK_MHZ must */     
    /* also be changed. */                           
    halBoardSetSystemClock(SYSCLK);            
  }                                                  
                                                                                                      
  /* CPU15 Errata workaround */                      
  __bic_SR_register(SCG0);                // Vorteiler = 1
}


/**************************************************************************************************
* @fn      halBoardSetSystemClock
*
* @brief   Configure DCO for system clock
*
* @param   systemClockSpeed
*
* @return  None
**************************************************************************************************/


void halBoardSetSystemClock(uint8 systemClockSpeed)
{
  uint8  setDcoRange, setVCore;
  uint16 setMultiplier;

  halBoardGetSystemClockSettings( systemClockSpeed, &setDcoRange, &setVCore, &setMultiplier);

  halBoardSetVCore( setVCore );
  UCSCTL0 = 0x00;                           /* Set lowest possible DCOx, MODx */
  UCSCTL1 = setDcoRange;                    /* Select suitable range */

  UCSCTL2 = setMultiplier + FLLD_1;         /* Set DCO Multiplier */
  UCSCTL4 = SELA__XT1CLK | SELS__DCOCLKDIV | SELM__DCOCLKDIV ;
}


/**************************************************************************************************
* @fn      halBoardGetSystemClockSettings
*
* @brief   Get the DCO configuration parameters
*
* @param    systemClockSpeed: range from 1 to 25MHz
*          *setDcoRange:      DCO select
*          *setVCore:         VCORE settings
*          *setMultiplier:    Multiplier settings
*
* @return  None
**************************************************************************************************/
void halBoardGetSystemClockSettings(uint8   systemClockSpeed,
                                    uint8  *setDcoRange,
                                    uint8  *setVCore,
                                    uint16 *setMultiplier)
{
  switch (systemClockSpeed)
  {
  case SYSCLK_1MHZ:
    *setDcoRange = DCORSEL_1MHZ;
    *setVCore = VCORE_1MHZ;
    *setMultiplier = DCO_MULT_1MHZ;
    break;
  case SYSCLK_4MHZ:
    *setDcoRange = DCORSEL_4MHZ;
    *setVCore = VCORE_4MHZ;
    *setMultiplier = DCO_MULT_4MHZ;
    break;
  case SYSCLK_8MHZ:
    *setDcoRange = DCORSEL_8MHZ;
    *setVCore = VCORE_8MHZ;
    *setMultiplier = DCO_MULT_8MHZ;
    break;
  case SYSCLK_12MHZ:
    *setDcoRange = DCORSEL_12MHZ;
    *setVCore = VCORE_12MHZ;
    *setMultiplier = DCO_MULT_12MHZ;
    break;
  case SYSCLK_16MHZ:
    *setDcoRange = DCORSEL_16MHZ;
    *setVCore = VCORE_16MHZ;
    *setMultiplier = DCO_MULT_16MHZ;
    break;
  case SYSCLK_18MHZ:
    *setDcoRange = DCORSEL_18MHZ;
    *setVCore = VCORE_18MHZ;
    *setMultiplier = DCO_MULT_18MHZ;
    break;
  case SYSCLK_20MHZ:
    *setDcoRange = DCORSEL_20MHZ;
    *setVCore = VCORE_20MHZ;
    *setMultiplier = DCO_MULT_20MHZ;
    break;
  case SYSCLK_25MHZ:
    *setDcoRange = DCORSEL_25MHZ;
    *setVCore = VCORE_25MHZ;
    *setMultiplier = DCO_MULT_25MHZ;
    break;
  }
}



/**************************************************************************************************
* @fn      halBoardSetVCoreUp
*
* @brief   Tuning Voltage up
*
* @param   level: VCORE setting level
*
* @return  None
**************************************************************************************************/
void halBoardSetVCoreUp (uint8 level)
{
  PMMCTL0_H = 0xA5;                         /* Open PMM module registers for write access */

  /* Set SVS/M high side to new level */
  SVSMHCTL = (SVSMHCTL & ~(SVSHRVL0*3 + SVSMHRRL0)) | (SVSHE + SVSHRVL0 * level + SVMHE + SVSMHRRL0 * level);
  SVSMLCTL = SVSLE + SVMLE + SVSMLRRL0 * level;  /* Set SVM new Level */
  SVSMLCTL = (SVSMLCTL & ~(SVSMLRRL_3)) | (SVMLE + SVSMLRRL0 * level); /* Set SVS/M low side to new level */
  while ((PMMIFG & SVSMLDLYIFG) == 0);      /* Wait till SVM is settled (Delay) */
  PMMCTL0_L = PMMCOREV0 * level;            /* Set VCore to x */
  PMMIFG &= ~(SVMLVLRIFG + SVMLIFG);        /* Clear already set flags */
  if ((PMMIFG & SVMLIFG))
  {
    while ((PMMIFG & SVMLVLRIFG) == 0);     /* Wait till level is reached */
  }

  /* Set SVS/M Low side to new level */
  SVSMLCTL = (SVSMLCTL & ~(SVSLRVL0*3 + SVSMLRRL_3)) | (SVSLE + SVSLRVL0 * level + SVMLE + SVSMLRRL0 * level);
  PMMCTL0_H = 0x00;                         /* Lock PMM module registers for write access */
}


/**************************************************************************************************
* @fn      halBoardSetVCoreDown
*
* @brief   Tuning Voltage down
*
* @param   level: VCORE setting level
*
* @return  None
**************************************************************************************************/
void halBoardSetVCoreDown (uint8 level)
{
  PMMCTL0_H = 0xA5;                         /* Open PMM module registers for write access */
  /* Set SVS/M low side to new level */
  SVSMLCTL = (SVSMLCTL & ~(SVSLRVL0*3 + SVSMLRRL_3)) | (SVSLRVL0 * level + SVMLE + SVSMLRRL0 * level);
  while ((PMMIFG & SVSMLDLYIFG) == 0);      /* Wait till SVM is settled (Delay) */
  PMMCTL0_L = (level * PMMCOREV0);          /* Set VCore to 1.85 V for Max Speed. */
  PMMCTL0_H = 0x00;                         /* Lock PMM module registers for write access */
}


/**************************************************************************************************
* @fn      halBoardSetVCore
*
* @brief   Setting up VCO
*
* @param   level: VCORE setting level
*
* @return  None
**************************************************************************************************/
void halBoardSetVCore (uint8 level)
{
  uint8 actLevel;

  do {
    actLevel = PMMCTL0_L & PMMCOREV_3;
    if (actLevel < level)
    {
      halBoardSetVCoreUp(++actLevel);       /* Set VCore (step by step) */
    }
    if (actLevel > level)
    {
      halBoardSetVCoreDown(--actLevel);     /* Set VCore (step by step) */
    }
  } while (actLevel != level);
}



/**************************************************************************************************
 * @fn      SPI_A0_Init
 *
 * @brief   Initilize SPI Connection
 *
 * @param   None
 *
 * @return  None
 **************************************************************************************************/

void SPI_A0_Init(void)
{
 
  P3SEL |= BIT3;                            /* P3.3 option select SIMO */
  P2SEL |= BIT7;                            /* P2.7 option select CLK */
  
  UCA0CTL1 |= UCSWRST;                      /* **Put state machine in reset**  */
  UCA0CTL0 |= UCMST+UCSYNC+UCCKPL+UCMSB;    /*3-pin, 8-bit SPI master  */
                                            /* Clock polarity high, MSB  */
  UCA0CTL1 |= UCSSEL_2;                     /* SMCLK  */
  UCA0BR0 = 0x02;                           /* /2  */
  UCA0BR1 = 0;                              
  UCA0MCTL = 0;                             /* No modulation */
  UCA0CTL1 &= ~UCSWRST;                     /* **Initialize USCI state machine**  */
 // UCA0IE |= UCRXIE;                         /* Enable USCI_A0 RX interrupt  */
}



/**************************************************************************************************
 * @fn      SPI_A0_Send_Data
 *
 * @brief   Sends a Data Array of i Bytes 
 *
 * @param   *Data:   pointer on data array
                i:   number of Bytes
 *
 * @return  None
 **************************************************************************************************/

void SPI_A0_Send_Data(uint8* DATA, uint8 i)
{
  while(i)
  {
    while (!(UCA0IFG & UCTXIFG));  /* USCI_A0 TX buffer ready?   */
    UCA0TXBUF = *DATA;              /* Transmit data   */
    DATA++;                         /* Increment the pointer on the array  */
    i--;                            /* Decrement the Byte counter  */
  }
  while (!(UCA0IFG & UCTXIFG));    /* Wait until USCI_A0 TX buffer is ready? */
}




/**************************************************************************************************
**************************************************************************************************/






